# 存储大家庭

## 什么是存储？
把特定的数据结构 转为 **可以被记录、可以被还原** 的格式。

注意，大家容易混淆的一个概念就是存储就是存到硬盘中，实际上，存到内存、通过网络传输也算是存储的一种形式，即数据序列化。

## 存储要考虑的点，不只是存取这么简单

存储的知识点可不仅仅包含 SharedPreferences、 mmkv、sqlite，这些只是实现工具，重要的是，要知道我们的数据：

### 1. 容错要求  
数据的正确性是一切的前提，考虑下使用场景，有没有多线程、跨进程同步操作，异常情况下数据的校验和恢复也属于重点设计，可以采用双写或者备份文件策略。
> tips: 备份文件，你可以在 SharedPreferences 的源码中窥见这一实现逻辑。

### 2. 以什么样的组织形式？以什么样的频率访问？  

我们想一想，以下所示的一个类，要如何保存？

```
class User{
  private int id = 1;
  private String name = "haha";
}
```

我们是不是可以这样

```
<class>
  <attribute name="className">User</attribute>
  <attribute name="id">1</attribute>
  <attribute name="name">haha</attribute>
</class>
```

可以这样
```
{"id":1,"name":"haha"}
```

甚至还可以这样

```
//格式为：数据长度紧挨着数据 注意 数据排放要按顺序 把一个bean做线性结构化
114haha
```

以上就是数据序列化的伪代码。

除了数据序列化，还可以采用对象序列化，如Serializable、Parcelable、Twitter的Serial开源库。把一个 Object 对象所有的信息表示成一个字节序列，包括 Class 信息、继承关系信息、访问权限、变量类型以及数值信息等。

当然，你会发现对比起数据序列化，对象序列化要记录的信息还是比较多，在操作比较频繁的时候，我们可以优先考虑使用前者。而且前者还有一个小优势：基本都是跨平台跨语言的。

### 3. 空间考量  
这里包含了内存开销和硬盘开销，一串数据就这么多，我们可以从哪里着手呢？  

1. 压缩策略，如 gzip；
    
2. 考虑其他编码格式，不同编码格式占用空间不同 xml>Json>pb
    
### 4. 时间考量   
前面我们说到可以压缩数据，那压缩势必会带来CPU处理的耗时。

1. 考虑CPU时间，编解码、加解密算法的优化空间
    
2. I/O时间，这就涉及到了I/O的知识点，是否能减少系统调用？是否能减少拷贝次数？
    
> tips: 多路复用、mmap

### 5. 安全性要求  
对称加密虽快，那密钥的保存和交换要如何设计？你一定会想到 非对称加密协商密钥，对称加密加密数据 的法子，还有其他法子吗？

### 6. 维护成本和业务接入成本

### 7. 兼容性
考不考虑向前兼容（老的数据在升级后可以迁移）和向后兼容（新的数据在老版本可以降级使用）？
        

## 存储大家庭如何学？

本模块没有先后顺序的强制，推荐使用**横向比较法**，何为横向比较？以上所论述的点，既是我们设计的考虑点，也是我们在考量评估一个存储工具时横向对比的点。梳理出各种不同存储方式的优缺点、使用场景、改造方式 就是初步的横向比较法。

### SharedPreferences
> 关键词：xml、全量更新、apply()ANR

- [反思｜官方也无力回天？Android SharedPreferences的设计与实现](https://juejin.cn/post/6884505736836022280)
> 相信你总是听到“sp不安全”、“sp性能差”，“要用mmkv”之类的云云，那 sp 真的无可救药了吗？或者说，难道官方就改不动了吗？本文从设计者的角度来剖析这个问题，精彩精彩！就算不是来学习 SharedPreferences，本文的写作手法也值得学习。反复地抛出疑问引起读者的好奇心，再一步步去探索答案，行文相当流畅。（是我一直想做的文章视角，好吧，我又鸽了）
> 
> 说句题外话，**源码+注释分析 的博客总结是非常差的一种学习方式**，因为这通常意味着**从结果倒推过程**，这往往导致被作者的思想牵着鼻子走。我建议，看之前要有自己的设想、疑问、预期，在阅读的过程中不断去比较自己的设想，重在思想的提炼。

- [面试高频题：一眼看穿 SharedPreferences](https://juejin.cn/post/6844903758355234824)
> 适合作为细细阅读 SharedPreferences 源码的辅助。

### MMKV
> 关键词：pb、增量更新、文件锁

### 文件

### 数据库
