# 数据结构与算法

### 前言

我认为数据结构更像高度抽象，学习过程中不必纠结于代码，从顶层宏观地去观察数据结构的各种操作。

### 入门篇
> 首先你得先知道数据结构是什么，先打造数据结构的知识体系网。

- [时间复杂度和空间复杂度](https://www.jianshu.com/p/bbcda16b2e90)
> 要学算法，我们必须先知道怎么度量算法的好坏，估算的衡量标准是什么，如何在代码层面快速看出时间复杂度，这些能帮助我们在日常的编程中更好地选取数据结构和算法。

- 《大话数据结构》
> 我接触数据结构时读的第一本书，深度广度不够，但是作为一本入门书已经足够了。

- 《算法 第四版》
> 用大量的图片来演示算法，比较友好。

- [小白一路走来，连续刷题三年，谈谈我的算法学习经验](https://www.cnblogs.com/kubidemanong/p/10996134.html)
> 本文是作者的刷题心路历程，该看什么书？该怎么刷题？有没有什么解题技巧？如果你有这些困惑，本文可以帮助你。

- [labuladong 的算法小抄](https://labuladong.gitbook.io/algo/)
> 只要在力扣解不开的题目，题解区有 labuladong 大佬，我就知道，这题我必会了，哈哈哈哈。刷算法是有套路的，labuladong 最强的地方在于，用最通俗的描述，最通用的框架来理解算法。

- 《算法导论》
- [网易公开课 —— 算法导论](http://open.163.com/newview/movie/courseintro?newurl=%2Fspecial%2Fopencourse%2Falgorithms.html)
> 觉得算法导论难啃，可以看看中文网易公开课

### 摆脱枯燥的文字，一起动起来？

- [visualgo](https://visualgo.net/zh)
> Steven Halim 博士在 2011 年发布的一款可视化学习算法的工具，用于帮助其学生更好地理解数据结构和算法，可以让学生按自己的步骤来学习。

- [LeetCodeAnimation](https://github.com/MisterBooo/LeetCodeAnimation)
> 用动画的形式呈现解LeetCode题目的思路，相当有趣

- [有趣生动的排序算法3D动画（转自YouTube）](https://www.bilibili.com/video/BV1x7411M7Zj?p=1)
> 机器小人莫名有点喜感怎么回事？！

### 快速回顾

- 《轻松学算法：互联网面试算法宝典》
> 适合有基础的小伙伴用来查漏补缺，讲解数据结构的部分偏基础，连带着介绍了一些常见的互联网面试题及分析思路，个人觉得本书精华在于：
> 1. 面试题的思路剖析讲解
> 2. 关注算法的具体实践应用，不仅有原理，还有适用场景和改进。（比如快排、冒泡排序的改进）

### 实战
> 算法能力的提升和做题的数量是有一定的关系（不是线性关系）。在做题时要力求一题多解，寻求最优解。

刷题目网站推荐
- [leetcode](https://leetcode.com/)
- [牛客网](https://www.nowcoder.com/activity/oj)

### 扩展

#### LRU、LFU

- [其实吧，LRU也就那么回事。](https://mp.weixin.qq.com/s?__biz=Mzg3NjU3NTkwMQ==&mid=2247505205&idx=1&sn=fde0899d41a62891d0a6ab50f2f7044b&source=41#wechat_redirect)
- [哎，这让人抠脑壳的 LFU。](https://mp.weixin.qq.com/s?__biz=Mzg3NjU3NTkwMQ==&mid=2247505209&idx=1&sn=26c92a5f7f1dc245f37a1aacc1e49990&source=41#wechat_redirect)
> 由浅入深，一一剖析，我强烈推荐每个小伙伴学数据结构时，可以自己 mock 业务场景出来练手，比如：
> 
> 想要实现：查询和插入的时间复杂度都是 O(1)，那么我们该如何选数据结构？  
> 答：需要一个满足下面三个条件的数据结构：  
>  1. 查询O(1)，考虑 HashMap  
>  2. 要不要有序？  
>  3. 插入O(1)，考虑链表 List，删除也要O(1)，不要费老大劲去找前驱节点，那就考虑双向链表 LinkedList  
> 好，留意到后面的单词了吗？我们把单词组合起来，LinkedHashMap没跑了。  


#### 位运算
- [真实世界的异或运算](https://mp.weixin.qq.com/s/pDXzZoSByzYKkim71qCsCw)
> 刷题的时候，常常遇到的位运算就是异或运算（XOR）了，比如找一个只出现一次的数字，原理很简单，实现也很简单，现实作用呢？  
> 在这篇文章中，将带我们深入到计算机底层，展示真实世界的异或运算是被怎样应用的：数字交换、编译器优化、清零、对称加密、更加节省空间的双向链表 —— 异或双向链表。

#### 排序

大家会不会觉得不同算法的 时间复杂度、空间复杂度 难记？**这不是记出来的，而是推导出来的。**

推荐你个小方法，一边看[动画](https://visualgo.net/zh/sorting)，一边自己推导出复杂度，再和实际结果做对比，看看自己是否对复杂度的推导有误。

另外，要多思考，当你在用数字集来参与排序时，稳定性有什么意义？是因为在生产环境中有什么需求吗？

这又衍生出了个扩展知识点，有意思吧！

- [如果我问你：排序算法的「稳定性」有何意义？你怎么回答？](https://www.lmonkey.com/t/87y4WkQE3)
> 你有没有留意过算法稳定性呢？

- [[译]理解timsort, 第一部分：适应性归并排序(Adaptive Mergesort)](http://blog.kongfy.com/2012/10/%E8%AF%91%E7%90%86%E8%A7%A3timsort-%E7%AC%AC%E4%B8%80%E9%83%A8%E5%88%86%EF%BC%9A%E9%80%82%E5%BA%94%E6%80%A7%E5%BD%92%E5%B9%B6%E6%8E%92%E5%BA%8Fadaptive-mergesort/)
> 通过一个简单的例子，告诉读者如何从一个最基本的归并排序开始逐步得到 timsort的“核心”:基本的适应性归并排序。

- [如何找出Timsort算法和玉兔月球车中的Bug？](https://www.freebuf.com/vuls/62129.html)
> Timsort排序算法在 Java SE 7, Android, Python, GNU Octave 中都得到了应用，这个工业级的算法，你不可错过。

- [单轴快排（SinglePivotQuickSort）和双轴快排（DualPivotQuickSort）及其JAVA实现](https://blog.csdn.net/holmofy/article/details/71168530)
> Arrays.sort源码中使用了改进快排 DualPivotQuickSort，单轴变双轴。

#### KMP

> 个人觉得，刷题不能仅仅为了刷题，而是要了解题目背后算法诞生的意义和应用，这样才能活学活用。比如：谈到字符串模式匹配算法，莫过于最经典的 KMP 算法。那字符串的模式匹配的应用在哪呢？在于生物信息学、信息检索、拼写检查、语言翻译、数据压缩、网络入侵检测等领域，那应用如此广泛，如何简化其复杂性就显得尤为必要了，这也是 KMP 诞生的背景。学着学着你会发现，不管是 KMP 也好，动态规划也罢，它们都是基于暴力法演变而来的，最后，记得用题目检验一下自己的吸收程度。（28. 实现 strStr()、686. 重复叠加字符串匹配、459. 重复的子字符串、214. 最短回文串）

- [【算法ABC】KMP算法](https://wnjxyk.tech/2638.html)
- [三步学通KMP](https://blog.csdn.net/helloworldchina/article/details/104465772)
